<?php
namespace Afone\ComNpay\Controller\Index;

use \Magento\Framework\App\Action\Context;
use \Magento\Framework\App\Action\Action;
use \Magento\Sales\Model\Order;
use \Magento\Catalog\Api\ProductRepositoryInterface;
use \Magento\CatalogInventory\Model\Stock\StockItemRepository;

class Ipn extends Action
{

  public function __construct(
    Context $context,
    ProductRepositoryInterface $productRepository,
    StockItemRepository $stockItemRepository
  ) 
  {
    $this->_productRepository = $productRepository;
    $this->_stockItemRepository = $stockItemRepository;
    parent::__construct(
        $context
    );
  }
  
  
  public function execute()
  {
    // Get the data
    $post = filter_input_array(INPUT_POST);
    $transactionId = $post["idTransaction"];
    $result = $post["result"];
    $advancedParameter = $post["advancedParameter"];
    $amountGateway = (float)$post["montant"];
    $explodeTransac = explode("-", $transactionId);

    // Get order
    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();

    // Get helper config (Data from admin ComNpay)
    $helper = $objectManager->create('Afone\ComNpay\Helper\Data');
    // Get secret key to verify IPN call
    $ipn_secret_key = $helper->getGeneralConfig('secret_key');


    $sec = $post['sec'];
    unset($post['sec']);
    if(strtoupper(hash("sha512", base64_encode(implode("|",$post)."|".$ipn_secret_key))) != strtoupper($sec))  {
      header("Status: 400 Bad Request", false, 400);
    }else{
      $order = $objectManager->create('\Magento\Sales\Model\Order')->load($explodeTransac[1]);
      $order_data = $order->getData(); 
      
      if($result == 'OK'){

        // If merchant chose to complete the order at the end of payment
        if($helper->getGeneralConfig('order_final_status')){
          $order->setState(Order::STATE_COMPLETE)->setStatus(Order::STATE_COMPLETE);
        }else{
          $order->setState(Order::STATE_PROCESSING)->setStatus(Order::STATE_PROCESSING);
        }
        
        $order->setCanSendNewEmailFlag(true);
        // Change paid amount to order total amount
        $order->setTotalPaid($amountGateway);
        // Change due amount to 0
        $order->setTotalDue(0);
        // Reduce quantity of items in stock
        $items = $order->getAllItems(); // Get all items from the order
        if ($items) {
            foreach($items as $item) {
                if ($item->getParentItem()) { 
                    continue;
                }
                    $quantity = $item->getQtyOrdered(); // Get quantity for each item
                    $product = $this->_productRepository->getById($item->getProductId());
                if($product){
                    $stock = $product->getExtensionAttributes()->getStockItem();
                    $stock->setQty($stock->getQty()-$quantity); // Set to new quantity     
                    $stock->save(); // Save
                }
            }
        }



        if($post['data'] == 'P3F'){
          $order->addStatusHistoryComment(__('IMPORTANT, this transaction is made with a payment in 3 installments'));
        }
        $order->save();
        $emailSender = $objectManager->create('\Magento\Sales\Model\Order\Email\Sender\OrderSender');
        $emailSender->send($order, true);
      }else if($advancedParameter != "RequestCompleted") {
        $order->setState(Order::STATE_CANCELED)->setStatus(Order::STATE_CANCELED);
        $order->save();
      }else if($advancedParameter == "RequestCompleted"){
          $order->addStatusHistoryComment(__('IMPORTANT, this SOISY subscription has been done, but not verified yet'));
      }
    }

    
    
    
    
  }

}