<?php
namespace Afone\ComNpay\Controller\Index;

use \Magento\Framework\App\Action\Context;
use \Magento\Framework\View\Result\PageFactory;
use \Magento\Framework\App\Action\Action;
use \Magento\Framework\Registry;
use \Magento\Catalog\Api\ProductRepositoryInterface;
use \Magento\CatalogInventory\Model\Stock\StockItemRepository;

class Redirect extends Action
{
  
  protected $_coreRegistry;
  protected $_pageFactory;
  
  public function __construct(
    Context $context,
    PageFactory $resultPageFactory,
    Registry $resultRegistry,
    ProductRepositoryInterface $productRepository,
    StockItemRepository $stockItemRepository
  ) 
  {
    $this->_pageFactory = $resultPageFactory;
    $this->_coreRegistry = $resultRegistry;
    $this->_productRepository = $productRepository;
    $this->_stockItemRepository = $stockItemRepository;
    parent::__construct(
        $context
    );
  }
    
  
  public function execute()
  {

    
    $get = filter_input_array(INPUT_GET);
    
    // Get Order by id
    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    $order = $objectManager->create('\Magento\Sales\Model\Order')->load(intval($get['id']));
    $order_data = $order->getData(); 
    if(!empty($order_data['customer_id'])){
      
      // Get Customer by id
      $customer = $objectManager->create('\Magento\Customer\Model\Customer')->load(intval($order_data['customer_id']));
      $customer_data = $customer->getData();


      // Get Address by id
      $address = $objectManager->create('\Magento\Customer\Model\Address')->load(intval($customer_data['default_shipping']));
      $address_data = $address->getData();
    }


    // Get helper config (Data from admin ComNpay)
    $helper = $objectManager->create('Afone\ComNpay\Helper\Data');

    // Get store config
    $store = $objectManager->get('\Magento\Framework\Locale\Resolver');
    $lang = explode("_", $store->getLocale());

    // Get BaseUrl
    $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
    $baseUrl = $storeManager->getStore()->getBaseUrl();


    // Create ComNpay inputs
    $comnpay['montant']         = number_format($order_data['grand_total'], 2 , '.', '');

    $comnpay['idTPE']           = $helper->getGeneralConfig('tpe_nb');

    $comnpay['idTransaction']   = time() . '-' . intval($get['id']);
    $comnpay['idCommande']      = intval($get['id']);
    $comnpay['devise']          = 'EUR';
    $comnpay['lang']            = $lang[0];
    
    $comnpay['nom_produit']     = $order_data['increment_id'];
    $comnpay['source']          = $baseUrl;
    $comnpay['urlIPN']          = $baseUrl . "afone_comnpay/index/ipn/";
    $comnpay['extension']       = "magento2-comnpay";
    if($get['pnf'] == 1){
      $comnpay['typeTr']          = "P3F";
      $comnpay['data']            = "P3F";
    }else{
      $comnpay['typeTr']          = "D";
      $comnpay['data']            = "D";
    }

    
    // Check if customer has id (to recover address) and redirect after payment in order view 
    if(!empty($order_data['customer_id'])){
      $comnpay['urlRetourOK']     = $baseUrl . "sales/order/view/order_id/" . $get['id'];
      $comnpay['urlRetourNOK']    = $baseUrl . "sales/order/view/order_id/" . $get['id'];
      $porteur = array(
        "nom"         => $order_data['customer_lastname'],
        "prenom"      => $order_data['customer_firstname'],
        "email"       => $order_data['customer_email'],
        "adresse"     => $address_data['street'],
        "codePostal"  => $address_data['postcode'],
        "ville"       => $address_data['city'],
        "telephone"   => $address_data['telephone'],
        "pays"        => $address_data['country_id'],
        "refPorteur"  => $order_data['customer_id']
      );
    }else{
      
      $comnpay['urlRetourOK']     = $baseUrl . "afone_comnpay/index/retourOk?oar_order_id=" . $order->getIncrementId() ."&oar_billing_lastname=" . $order->getBillingAddress()->getLastname() . "&oar_type=email&oar_email=" . $order_data['customer_email'] . "&result=OK";
      $comnpay['urlRetourNOK']     = $baseUrl . "afone_comnpay/index/retourOk?oar_order_id=" . $order->getIncrementId() ."&oar_billing_lastname=" . $order->getBillingAddress()->getLastname() . "&oar_type=email&oar_email=" . $order_data['customer_email'] . "&result=KO&id_simple=" . $get['id'];
      $porteur = array(
        "nom"         => $order->getBillingAddress()->getLastName(),
        "prenom"      => $order->getBillingAddress()->getFirstName()
      );
    }


    
    $comnpay['porteur']         = base64_encode(json_encode($porteur));

    $comnpay['key']           = $helper->getGeneralConfig('secret_key');


    // Encoding
    $comnpayWithKey = base64_encode(implode("|", $comnpay));
    unset($comnpay['key']);
    $comnpay['sec'] = hash("sha512",$comnpayWithKey);
    
    
    // Save data in register
    $this->_coreRegistry->register('data_comnpay', json_encode($comnpay));

    // Get server (homologation or production)
    if($helper->getGeneralConfig('homologation')){
      $urlPsp = "https://secure-homologation.comnpay.com";
    }else{
      $urlPsp = "https://secure.comnpay.com";
    }
    $this->_coreRegistry->register('urlPsp', $urlPsp);

    // Don't reduce quantity of items in stock
    $items = $order->getAllItems(); // Get all items from the order
    if ($items) {
        foreach($items as $item) {
            if ($item->getParentItem()) { 
                continue;
            }
                $quantity = $item->getQtyOrdered(); // Get quantity for each item
                $product = $this->_productRepository->getById($item->getProductId());
            if($product){
                $stock = $product->getExtensionAttributes()->getStockItem();
                $stock->setQty($stock->getQty()+$quantity); // Set to old quantity
                $stock->save(); // Save
            }
        }
    }
    
    
    // Call layout.xml
    $resultPage = $this->_pageFactory->create();
    $resultPage->addHandle('comnpay_index_redirect');
    $resultPage->getConfig()->getTitle()->prepend(__('Creating payment'));
    return $resultPage;
    
  }
}